#include "stdafx.h"
#include "SamplePlugin.h"
#include "Common/FileRecord.h"
#include "resource.h"

#include <shlwapi.h>

#pragma comment(lib,"shlwapi.lib")

static CSamplePlugin *m_pSelf = NULL;

static INT_PTR CALLBACK DialogProc(HWND hWnd, UINT msg, WPARAM wp, LPARAM lp)
{
    if (msg == WM_INITDIALOG) {
	m_pSelf = (CSamplePlugin *)lp;
	return TRUE;
    } else
	if (m_pSelf != NULL)
	    return m_pSelf->DialogProc(hWnd, msg, wp, lp);

    return FALSE;
}

CSamplePlugin::CSamplePlugin(HMODULE hModule)
{
    m_hModule = hModule;

	m_iServiceID = 0;
	m_TsStream.SetDecodePSI(true);
	m_TsStream.SetEpgProc(false);
	m_TsStream.SetCallbackFunc(TsCallback,this);
	ZeroMemory(&m_stStreamInfo,sizeof(struct TS_StreamInfo));
}

BOOL CSamplePlugin::Init(HWND AppWindow)
{
    m_hWnd = CreateDialogParam(m_hModule, MAKEINTRESOURCE(IDD_SAMPLE), AppWindow, ::DialogProc, (LPARAM)this);
	CheckDlgButton(m_hWnd,IDC_AUTOSCROLL,BST_CHECKED);

    return TRUE;
}

BOOL CSamplePlugin::ProcessTS(BYTE *Buffer, DWORD Length)
{
	m_TsStream.OnTsStream(Buffer,Length,&m_stStreamInfo);

    return TRUE;
}

BOOL CSamplePlugin::SetHostAPI(IDTVApp *Application, DWORD Version)
{
    // please make sure to include this check
    // when IDTVApp interface is updated, SDK_MAJOR version will be incremented
    // when only simple updates to SDK are done, only minor version is incremented
    // ignoring this check will crash your plugin if interface changes
    if ((FRIIO_SDK_MAJOR(Version) != FRIIO_SDK_MAJOR(FRIIO_SDK_VERSION)) ||
	(FRIIO_SDK_RELEASE(Version) != FRIIO_SDK_RELEASE(FRIIO_SDK_VERSION))) {
	MessageBox(NULL, _T("Wrong SDK Version"), _T("Error"), MB_OK | MB_ICONSTOP);
	return FALSE;
    }

    m_pIDtvApp = Application;

    return TRUE;
}

BOOL CSamplePlugin::OnStreamChange(int Channel, int ServiceID)
{ 
    CDTVProgram progs[16] = { 0, };
    int count = 16;
	m_iServiceID = ServiceID;

    if (!m_pIDtvApp->GetProgramList(progs, &count))
	return TRUE;

    if (count < 1)
	return TRUE;

    return TRUE;
}

BOOL CSamplePlugin::OnFileSave(TCHAR *FileName, DWORD Length)
{
    // Host app passes entire path to file about to be saved.
    // Plugin can either change the path/filename and return TRUE to start recording,
    // or return FALSE to interrupt recording.
    return TRUE;
}

INT_PTR CSamplePlugin::DialogProc(HWND hWnd, UINT msg, WPARAM wp, LPARAM lp)
{
    switch (msg) {
	case WM_INITDIALOG:
		{			
			return TRUE;
		}
	    break;

	case WM_DESTROY:
		{
			m_hWnd = NULL;
		}
		break;

	case WM_COMMAND:
	    switch (LOWORD(wp)) {
		case IDC_SAVE:
			{
				char szPath[MAX_PATH];
				if(!CFileRecord::GetPathName(m_hWnd,szPath,MAX_PATH)) return FALSE;
				ServiceMap::iterator it=m_Service.find((WORD)m_iServiceID);
				if(it!=m_Service.end())
				{
					char szAppendDir[MAX_PATH];
					long year,month,day,hour,min,sec;
					CDateTime::ConvertFrom_MJD(CDateTime::GetCurrentDateTime().Date,year,month,day);
					CDateTime::ConvertFrom_TimeSec(CDateTime::GetCurrentDateTime().Time,hour,min,sec);
					wsprintfA(szAppendDir,"%d-%04d%02d%02d-%02d%02d%02d",m_iServiceID,year,month,day,hour,min,sec,hour,min,sec);
					PathAppendA(szPath,szAppendDir);
					PidList &List=it->second;
					for(unsigned int i=0;i<List.size();i++)
					{
						m_TsStream.OutputDataContent(szPath,List[i]);
					}
					AddString(L"t@Cۑ");
				}
			}
		    break;
	    }
	    break;
    }
    return FALSE;
}

void CSamplePlugin::AddString(wchar_t *str)
{
	if(m_hWnd)
	{
		LRESULT count = SendDlgItemMessage(m_hWnd,IDC_INFO,LB_GETCOUNT,0,0);
		LRESULT pos = SendDlgItemMessage(m_hWnd,IDC_INFO,LB_INSERTSTRING,count,(LPARAM)str);
		if(pos==LB_ERR) return;
		if(IsDlgButtonChecked(m_hWnd,IDC_AUTOSCROLL)==BST_CHECKED)
		{
			SendDlgItemMessage(m_hWnd,IDC_INFO,LB_SETCURSEL,pos,0);
		}
		if(count>1000)
		{
			SendDlgItemMessage(m_hWnd,IDC_INFO,LB_DELETESTRING,0,0);
		}
	}
}

void CSamplePlugin::TsCallback(UINT uType,WORD wStreamID,LPVOID pData,DWORD dwVal,LPVOID lpContext)
{
	wchar_t buf[256];
	CSamplePlugin *pThis = (CSamplePlugin*)lpContext;
	switch(uType)
	{
	case TSENG_CALLBACK_STREAM_RESET:
		{
			wsprintf(buf,L"Xg[ύX");			
			pThis->AddString(buf);
			pThis->m_Service.clear();
		}
		break;
	case TSENG_CALLBACK_PMT_RECIEVE:
		{
			wsprintf(buf,L"PMTM(Service %d)",wStreamID);
			pThis->AddString(buf);
			struct TAG_TSCB_PMTITEM *pstPmtItem=(struct TAG_TSCB_PMTITEM *)pData;
			ServiceMap::iterator it=pThis->m_Service.find(wStreamID);
			if(it==pThis->m_Service.end())
			{
				PidList vList;
				pThis->m_Service.insert(ServicePair(wStreamID,vList));
				it=pThis->m_Service.find(wStreamID);
				if(it==pThis->m_Service.end()) return;
			}
			PidList &List=it->second;

			List.clear();
			for(unsigned int i=0;i<dwVal;i++)
			{				
				wsprintf(buf,L"@PID=0x%04X Type=0x%02X",pstPmtItem[i].wPID,pstPmtItem[i].byType);
				pThis->AddString(buf);
				if(pstPmtItem[i].byType==0x0D)
				{
					// f[^Xg[̂
					List.push_back(pstPmtItem[i].wPID);
				}
			}
		}
	case TSENG_CALLBACK_DATA_COMPONENT_CLEAR:
		{
			WORD wPID=(WORD)wStreamID;
			BYTE byComponentTag = (BYTE)pData;			
			wsprintf(buf,L"R|[lgNA@PID=0x%04X ComponentTag=0x%02X",wPID,byComponentTag);
			pThis->AddString(buf);
		}
		break;		
	case TSENG_CALLBACK_DATA_MODULE_UPDATE:
		{
			WORD wPID=(WORD)wStreamID;
			struct TAG_TSCB_MODULE *pstModule = (struct TAG_TSCB_MODULE*)pData;
			wsprintf(buf,L"W[XV@PID=0x%04X ComponentTag=0x%02X ModuleID=0x%04X Ver=0x%02X  Size=%6d",
				wPID,pstModule->byComponentTag,pstModule->wModuleID,pstModule->byVersion,pstModule->dwSize);
			pThis->AddString(buf);
		}
		break;
	case TSENG_CALLBACK_DATA_MODULE_DELETE:
		{
			WORD wPID=(WORD)wStreamID;
			struct TAG_TSCB_MODULE *pstModule = (struct TAG_TSCB_MODULE*)pData;
			wsprintf(buf,L"W[폜@PID=0x%04X ComponentTag=0x%02X ModuleID=0x%04X Ver=0x%02X  Size=%6d",
				wPID,pstModule->byComponentTag,pstModule->wModuleID,pstModule->byVersion,pstModule->dwSize);
			pThis->AddString(buf);
		}
		break;
	case TSENG_CALLBACK_DATA_MODULE_COMPLETE:
		{
			WORD wPID=(WORD)wStreamID;
			struct TAG_TSCB_MODULE *pstModule = (struct TAG_TSCB_MODULE*)pData;
			wsprintf(buf,L"W[_E[h@PID=0x%04X ComponentTag=0x%02X ModuleID=0x%04X Ver=0x%02X  Size=%6d",
				wPID,pstModule->byComponentTag,pstModule->wModuleID,pstModule->byVersion,pstModule->dwSize);
			pThis->AddString(buf);
		}
		break;
	case TSENG_CALLBACK_DATA_COMPONENT_COMPLETE:
		{
			WORD wPID=(WORD)wStreamID;
			BYTE byComponentTag = (BYTE)pData;
			wsprintf(buf,L"R|[lg_E[h@PID=0x%04X ComponentTag=0x%02X",wPID,byComponentTag);
			pThis->AddString(buf);
		}
		break;		
	}
}